/*
 * ARISTOSTLE UNIVERSITY OF THESSALONIKI
 * Copyright (C) 2015
 * Aristotle University of Thessaloniki
 * Department of Electrical & Computer Engineering
 * Division of Electronics & Computer Engineering
 * Intelligent Systems & Software Engineering Lab
 *
 * Project             : WSAT
 * WorkFile            : 
 * Compiler            : 
 * File Description    : 
 * Document Description: 
* Related Documents	   : 
* Note				   : 
* Programmer		   : RESTful MDE Engine created by Christoforos Zolotas
* Contact			   : christopherzolotas@issel.ee.auth.gr
*/


package eu.fp7.scase.wsat.inputparameter;


import javax.ws.rs.core.UriInfo;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Response;
import com.sun.jersey.core.util.Base64;
import eu.fp7.scase.wsat.account.JavaaccountModel;

import eu.fp7.scase.wsat.utilities.HypermediaLink;
import eu.fp7.scase.wsat.utilities.HibernateController;

/* This class processes GET requests for inputparameter resources and creates the hypermedia to be returned to the client*/
public class GetinputparameterinputparameterHandler{


    private HibernateController oHibernateController;
    private UriInfo oApplicationUri; //Standard datatype that holds information on the URI info of this request
    private JavainputparameterModel oJavainputparameterModel;
	private String authHeader;
	private JavaaccountModel oAuthenticationAccount;

    public GetinputparameterinputparameterHandler(String authHeader, int inputparameterId, UriInfo oApplicationUri){
        oJavainputparameterModel = new JavainputparameterModel();
        oJavainputparameterModel.setinputparameterId(inputparameterId);
        this.oHibernateController = HibernateController.getHibernateControllerHandle();
        this.oApplicationUri = oApplicationUri;
		this.authHeader = authHeader;
		this.oAuthenticationAccount = new JavaaccountModel(); 
    }

    public JavainputparameterModel getJavainputparameterModel(){

    	//check if there is a non null authentication header
    	if(authHeader == null){
    		throw new WebApplicationException(Response.Status.FORBIDDEN);
    	}
		else{
	    	//decode the auth header
    		decodeAuthorizationHeader();

        	//authenticate the user against the database
        	oAuthenticationAccount = oHibernateController.authenticateUser(oAuthenticationAccount);

			//check if the authentication failed
			if(oAuthenticationAccount == null){
        		throw new WebApplicationException(Response.Status.UNAUTHORIZED);
        	}
		}

        return createHypermedia(oHibernateController.getinputparameter(oJavainputparameterModel));
    }

	/* This function performs the decoding of the authentication header */
    public void decodeAuthorizationHeader()
    {
    	//check if this request has basic authentication
    	if( !authHeader.contains("Basic "))
    	{
    		throw new WebApplicationException(Response.Status.BAD_REQUEST);
    	}
    	
        authHeader = authHeader.substring("Basic ".length());
        String[] decodedHeader;
        decodedHeader = Base64.base64Decode(authHeader).split(":");
        
        if( decodedHeader == null)
        {
        	throw new WebApplicationException(Response.Status.BAD_REQUEST);
        }
        
        oAuthenticationAccount.setusername(decodedHeader[0]);
        oAuthenticationAccount.setpassword(decodedHeader[1]);
    }

    /* This function produces hypermedia links to be sent to the client so as it will be able to forward the application state in a valid way.*/
    public JavainputparameterModel createHypermedia(JavainputparameterModel oJavainputparameterModel){

        /* Create hypermedia links towards this specific inputparameter resource. These can be GET, PUT and/or delete depending on what was specified in the service CIM.*/
        oJavainputparameterModel.getlinklist().add(new HypermediaLink(String.format("%s%s", oApplicationUri.getBaseUri(), oApplicationUri.getPath()), "Get the inputparameter", "GET", "Sibling"));
        oJavainputparameterModel.getlinklist().add(new HypermediaLink(String.format("%s%s", oApplicationUri.getBaseUri(), oApplicationUri.getPath()), "Update the inputparameter", "PUT", "Sibling"));
        oJavainputparameterModel.getlinklist().add(new HypermediaLink(String.format("%s%s", oApplicationUri.getBaseUri(), oApplicationUri.getPath()), "Delete the inputparameter", "DELETE", "Sibling"));

        /* Calculate the relative path towards any related resources of this one. Then add each new hypermedia link with that relative path to the hypermedia linklist to be sent back to client.*/
        String oRelativePath;
        oRelativePath = oApplicationUri.getPath().substring(oApplicationUri.getPath().indexOf("/inputparameter") + 1);
        oJavainputparameterModel.getlinklist().add(new HypermediaLink(String.format("%s%s/%s/%s", oApplicationUri.getBaseUri(), "multiinputparameterManager", oRelativePath, "inputparameter"), "Get all the inputparameters of this inputparameter", "GET", "Child"));
        oJavainputparameterModel.getlinklist().add(new HypermediaLink(String.format("%s%s/%s/%s", oApplicationUri.getBaseUri(), "multiinputparameterManager", oRelativePath, "inputparameter"), "Create a new inputparameter for this inputparameter", "POST", "Child"));

        /* Finally, truncate the current URI so as to point to the resource manager of which this resource is related.
        Then create the hypermedia links towards the parent resources.*/
        int iLastSlashIndex = String.format("%s%s", oApplicationUri.getBaseUri(), String.format("%s", oApplicationUri.getPath()).replaceAll("multiinputparameter","multiinputparameterManager")).lastIndexOf("/");
        oJavainputparameterModel.getlinklist().add(new HypermediaLink(String.format("%s%s", oApplicationUri.getBaseUri(), String.format("%s", oApplicationUri.getPath()).replaceAll("multiinputparameter","multiinputparameterManager")).substring(0, iLastSlashIndex), "Create a new inputparameter", "POST", "Parent"));
        oJavainputparameterModel.getlinklist().add(new HypermediaLink(String.format("%s%s", oApplicationUri.getBaseUri(), String.format("%s", oApplicationUri.getPath()).replaceAll("multiinputparameter","multiinputparameterManager")).substring(0, iLastSlashIndex), "Get all inputparameter of this inputparameter", "GET", "Parent"));
        return oJavainputparameterModel;
    }
}
