/*
 * ARISTOSTLE UNIVERSITY OF THESSALONIKI
 * Copyright (C) 2015
 * Aristotle University of Thessaloniki
 * Department of Electrical & Computer Engineering
 * Division of Electronics & Computer Engineering
 * Intelligent Systems & Software Engineering Lab
 *
 * Project             : WSAT
 * WorkFile            : 
 * Compiler            : 
 * File Description    : 
 * Document Description: 
* Related Documents	   : 
* Note				   : 
* Programmer		   : RESTful MDE Engine created by Christoforos Zolotas
* Contact			   : christopherzolotas@issel.ee.auth.gr
*/


package eu.fp7.scase.wsat.outputparameter;


import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.persistence.CollectionTable;
import javax.persistence.ElementCollection;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.JoinColumn;
import javax.persistence.OneToMany;
import org.hibernate.annotations.OnDelete;
import org.hibernate.annotations.OnDeleteAction;
import javax.persistence.Table;
import javax.persistence.Transient;

import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlTransient;

import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.annotations.ForeignKey;

import eu.fp7.scase.wsat.utilities.HypermediaLink;
import eu.fp7.scase.wsat.outputmessage.JavaoutputmessageModel;

import org.hibernate.search.annotations.Analyze;
import org.hibernate.search.annotations.Field;
import org.hibernate.search.annotations.FieldBridge;
import org.hibernate.search.annotations.Index;
import org.hibernate.search.annotations.Indexed;
import org.hibernate.search.annotations.IndexedEmbedded;
import org.hibernate.search.annotations.Store;

import eu.fp7.scase.wsat.utilities.SetStringFieldBridge;

/* This class models the data of a outputparameter resource. It is enhanced with JAXB annotations for automated representation
parsing/marshalling as well as with Hibernate annotations for ORM transformations.*/
@XmlRootElement
@Entity
@Table(name="outputparameter")
@Indexed
public class JavaoutputparameterModel{


    /* There follows a list with the properties that model the outputparameter resource, as prescribed in the service CIM*/
		@Column(name = "type")
		private String type;

		@Column(name = "name")
		private String name;

		@ElementCollection(fetch = FetchType.EAGER)
		@CollectionTable(name="outputparameterkeywords", joinColumns=@JoinColumn(name="outputparameterId"))
		@Column(name = "keywords")
		@ForeignKey(name = "fk_outputparameter_keywords")
		@Field(index=Index.YES, analyze=Analyze.YES, store=Store.NO)
		@FieldBridge(impl=SetStringFieldBridge.class)
		private Set<String> keywords = new HashSet<String>();

		@Id
		@GeneratedValue
		@Column(name = "outputparameterId")
		private int outputparameterId;

		// The Linklist property holds all the hypermedia links to be sent back to the client
		@Transient
		private List<HypermediaLink> linklist = new ArrayList<HypermediaLink>();

		// This property models the Many to One relationship between two resources as it is defined by the Hibernate syntax below.
		@ManyToOne(fetch = FetchType.EAGER)
		@JoinColumn(name="outputmessageId")
		@ForeignKey(name = "fk_outputmessage_outputparameter")
		private JavaoutputmessageModel outputmessage;

		// This property models the Many to One relationship between two resources as it is defined by the Hibernate syntax below.
		@ManyToOne(fetch = FetchType.EAGER)
		@JoinColumn(name="sourseoutputparameterId")
		@ForeignKey(name = "fk_outputparameter_outputparameter")
		private JavaoutputparameterModel outputparameter;

		// This property models the One to Many relationship between two resources as it is defined by the Hibernate syntax below.
		@OneToMany(fetch = FetchType.EAGER, mappedBy="outputparameter",orphanRemoval=true)
		@OnDelete(action=OnDeleteAction.CASCADE)
		private Set<JavaoutputparameterModel> SetOfJavaoutputparameterModel = new HashSet<JavaoutputparameterModel>();

    /* There follows a list of setter and getter functions.*/
	    public void settype(String type){
        	this.type = type;
    	}

	    public void setname(String name){
        	this.name = name;
    	}

	    public void setkeywords(Set<String> keywords){
        	this.keywords = keywords;
    	}

	    public void setoutputparameterId(int outputparameterId){
        	this.outputparameterId = outputparameterId;
    	}

	    public void setlinklist(List<HypermediaLink> linklist){
        	this.linklist = linklist;
    	}

		@XmlTransient
	    public void setoutputmessage(JavaoutputmessageModel outputmessage){
        	this.outputmessage = outputmessage;
    	}

		@XmlTransient
	    public void setoutputparameter(JavaoutputparameterModel outputparameter){
        	this.outputparameter = outputparameter;
    	}

		@XmlTransient
	    public void setSetOfJavaoutputparameterModel(Set<JavaoutputparameterModel> SetOfJavaoutputparameterModel){
        	this.SetOfJavaoutputparameterModel = SetOfJavaoutputparameterModel;
    	}

	    public String gettype(){
        	return this.type;
    	}

	    public String getname(){
        	return this.name;
    	}

	    public Set<String> getkeywords(){
        	return this.keywords;
    	}

	    public int getoutputparameterId(){
        	return this.outputparameterId;
    	}

	    public List<HypermediaLink> getlinklist(){
        	return this.linklist;
    	}

	    public JavaoutputmessageModel getoutputmessage(){
        	return this.outputmessage;
    	}

	    public JavaoutputparameterModel getoutputparameter(){
        	return this.outputparameter;
    	}

	    public Set<JavaoutputparameterModel> getSetOfJavaoutputparameterModel(){
        	return this.SetOfJavaoutputparameterModel;
    	}


    /* This function deletes explicitly any collections of this resource that are stored in the database 
    and iteratively does so for any subsequent related resources.
    NOTE: this function is needed to handle erroneous handling of cascade delete of some hibernate versions.*/
    public void deleteAllCollections(Session hibernateSession){

        Query query = hibernateSession.createSQLQuery(String.format("DELETE FROM %s where %sId = %d","outputparameterkeywords".toLowerCase(),"outputparameter",this.getoutputparameterId()));
        query.executeUpdate();

        Iterator<JavaoutputparameterModel> JavaoutputparameterModelIterator = SetOfJavaoutputparameterModel.iterator();
        while(JavaoutputparameterModelIterator.hasNext()){
            JavaoutputparameterModelIterator.next().deleteAllCollections(hibernateSession);
        }
    }
}
