/*
 * ARISTOSTLE UNIVERSITY OF THESSALONIKI
 * Copyright (C) 2015
 * Aristotle University of Thessaloniki
 * Department of Electrical & Computer Engineering
 * Division of Electronics & Computer Engineering
 * Intelligent Systems & Software Engineering Lab
 *
 * Project             : WSAT
 * WorkFile            : 
 * Compiler            : 
 * File Description    : 
 * Document Description: 
* Related Documents	   : 
* Note				   : 
* Programmer		   : RESTful MDE Engine created by Christoforos Zolotas
* Contact			   : christopherzolotas@issel.ee.auth.gr
*/


package eu.fp7.scase.wsat.restparameter;


import javax.ws.rs.core.UriInfo;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Response;
import com.sun.jersey.core.util.Base64;
import eu.fp7.scase.wsat.account.JavaaccountModel;

import eu.fp7.scase.wsat.utilities.HypermediaLink;
import eu.fp7.scase.wsat.utilities.HibernateController;
import eu.fp7.scase.wsat.restmethod.JavarestmethodModel;

/* This class processes POST requests for restparameter resources and creates the hypermedia to be returned to the client*/

public class PostrestmethodrestparameterHandler{


    private HibernateController oHibernateController;
    private UriInfo oApplicationUri; //Standard datatype that holds information on the URI info of this request
    private JavarestparameterModel oJavarestparameterModel;
    private JavarestmethodModel oJavarestmethodModel;
	private String authHeader;
	private JavaaccountModel oAuthenticationAccount;

    public PostrestmethodrestparameterHandler(String authHeader, int restmethodId, JavarestparameterModel oJavarestparameterModel, UriInfo oApplicationUri){
        this.oJavarestparameterModel = oJavarestparameterModel;
        this.oHibernateController = HibernateController.getHibernateControllerHandle();
        this.oApplicationUri = oApplicationUri;
        oJavarestmethodModel = new JavarestmethodModel();
        oJavarestmethodModel.setrestmethodId(restmethodId);
        oJavarestparameterModel.setrestmethod(this.oJavarestmethodModel);
		this.authHeader = authHeader;
		this.oAuthenticationAccount = new JavaaccountModel(); 
    }

    public JavarestparameterModel postJavarestparameterModel(){

    	//check if there is a non null authentication header
    	if(authHeader == null){
    		throw new WebApplicationException(Response.Status.FORBIDDEN);
    	}
		else{
	    	//decode the auth header
    		decodeAuthorizationHeader();

        	//authenticate the user against the database
        	oAuthenticationAccount = oHibernateController.authenticateUser(oAuthenticationAccount);

			//check if the authentication failed
			if(oAuthenticationAccount == null){
        		throw new WebApplicationException(Response.Status.UNAUTHORIZED);
        	}
		}

        return createHypermedia(oHibernateController.postrestparameter(oJavarestparameterModel));
    }

	/* This function performs the decoding of the authentication header */
    public void decodeAuthorizationHeader()
    {
    	//check if this request has basic authentication
    	if( !authHeader.contains("Basic "))
    	{
    		throw new WebApplicationException(Response.Status.BAD_REQUEST);
    	}
    	
        authHeader = authHeader.substring("Basic ".length());
        String[] decodedHeader;
        decodedHeader = Base64.base64Decode(authHeader).split(":");
        
        if( decodedHeader == null)
        {
        	throw new WebApplicationException(Response.Status.BAD_REQUEST);
        }
        
        oAuthenticationAccount.setusername(decodedHeader[0]);
        oAuthenticationAccount.setpassword(decodedHeader[1]);
    }

    /* This function produces hypermedia links to be sent to the client so as it will be able to forward the application state in a valid way.*/
    public JavarestparameterModel createHypermedia(JavarestparameterModel oJavarestparameterModel){

        /* Create hypermedia links towards this specific restparameter resource. These must be GET and POST as it is prescribed in the meta-models.*/
        oJavarestparameterModel.getlinklist().add(new HypermediaLink(String.format("%s%s", oApplicationUri.getBaseUri(), oApplicationUri.getPath()), "Get all restparameters of this restmethod", "GET", "Sibling"));
        oJavarestparameterModel.getlinklist().add(new HypermediaLink(String.format("%s%s", oApplicationUri.getBaseUri(), oApplicationUri.getPath()), "Create a new restparameter", "POST", "Sibling"));

        /* Then calculate the relative path to any resources that are related of this one and add the according hypermedia links to the Linklist.*/
        String oRelativePath;
        oRelativePath = oApplicationUri.getPath().replaceAll("multirestparameterManager/", "multirestparameter/");
        oJavarestparameterModel.getlinklist().add(new HypermediaLink(String.format("%s%s/%d", oApplicationUri.getBaseUri(), oRelativePath, oJavarestparameterModel.getrestparameterId()), oJavarestparameterModel.getparametername(), "GET", "Child", oJavarestparameterModel.getrestparameterId()));
        oJavarestparameterModel.getlinklist().add(new HypermediaLink(String.format("%s%s/%d", oApplicationUri.getBaseUri(), oRelativePath, oJavarestparameterModel.getrestparameterId()), oJavarestparameterModel.getparametername(), "PUT", "Child", oJavarestparameterModel.getrestparameterId()));
        oJavarestparameterModel.getlinklist().add(new HypermediaLink(String.format("%s%s/%d", oApplicationUri.getBaseUri(), oRelativePath, oJavarestparameterModel.getrestparameterId()), oJavarestparameterModel.getparametername(), "DELETE", "Child", oJavarestparameterModel.getrestparameterId()));

        /* Finally, calculate the relative path towards the resources of which this one is related.
        Then add hypermedia links for each one of them*/
        this.oJavarestmethodModel = HibernateController.getHibernateControllerHandle().getrestmethod(this.oJavarestmethodModel);
        oRelativePath = String.format("resource/%d/%s", this.oJavarestmethodModel.getresource().getresourceId(), oApplicationUri.getPath().replaceAll("multirestparameterManager/", ""));
        int iLastSlashIndex = String.format("%s%s", oApplicationUri.getBaseUri(), oRelativePath).lastIndexOf("/");
        oJavarestparameterModel.getlinklist().add(new HypermediaLink(String.format("%s%s", oApplicationUri.getBaseUri(), oRelativePath).substring(0, iLastSlashIndex), "Delete the parent restmethod", "DELETE", "Parent"));
        oJavarestparameterModel.getlinklist().add(new HypermediaLink(String.format("%s%s", oApplicationUri.getBaseUri(), oRelativePath).substring(0, iLastSlashIndex), "Get the parent restmethod", "GET", "Parent"));
        oJavarestparameterModel.getlinklist().add(new HypermediaLink(String.format("%s%s", oApplicationUri.getBaseUri(), oRelativePath).substring(0, iLastSlashIndex), "Update the restmethod", "PUT", "Parent"));
        return oJavarestparameterModel;
    }
}
