/*
 * ARISTOSTLE UNIVERSITY OF THESSALONIKI
 * Copyright (C) 2015
 * Aristotle University of Thessaloniki
 * Department of Electrical & Computer Engineering
 * Division of Electronics & Computer Engineering
 * Intelligent Systems & Software Engineering Lab
 *
 * Project             : WSAT
 * WorkFile            : 
 * Compiler            : 
 * File Description    : 
 * Document Description: 
* Related Documents	   : 
* Note				   : 
* Programmer		   : RESTful MDE Engine created by Christoforos Zolotas
* Contact			   : christopherzolotas@issel.ee.auth.gr
*/


package eu.fp7.scase.wsat.restparameter;


import javax.ws.rs.core.UriInfo;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Response;
import com.sun.jersey.core.util.Base64;
import eu.fp7.scase.wsat.account.JavaaccountModel;

import eu.fp7.scase.wsat.utilities.HypermediaLink;
import eu.fp7.scase.wsat.utilities.HibernateController;
import eu.fp7.scase.wsat.resource.JavaresourceModel;

/* This class processes PUT requests for restparameter resources and creates the hypermedia to be returned to the client*/
public class PutresourcerestparameterHandler{


    private HibernateController oHibernateController;
    private UriInfo oApplicationUri; //Standard datatype that holds information on the URI info of this request
    private JavarestparameterModel oJavarestparameterModel;
    private JavaresourceModel oJavaresourceModel;
	private String authHeader;
	private JavaaccountModel oAuthenticationAccount;


    public PutresourcerestparameterHandler(String authHeader, int resourceId, int restparameterId, JavarestparameterModel oJavarestparameterModel, UriInfo oApplicationUri){
        this.oJavarestparameterModel = oJavarestparameterModel;
        this.oJavarestparameterModel.setrestparameterId(restparameterId);
        this.oHibernateController = HibernateController.getHibernateControllerHandle();
        this.oApplicationUri = oApplicationUri;
        oJavaresourceModel = new JavaresourceModel();
        oJavaresourceModel.setresourceId(resourceId);
        oJavarestparameterModel.setresource(this.oJavaresourceModel);
		this.authHeader = authHeader;
		this.oAuthenticationAccount = new JavaaccountModel(); 
    }

    public JavarestparameterModel putJavarestparameterModel(){

    	//check if there is a non null authentication header
    	if(authHeader == null){
    		throw new WebApplicationException(Response.Status.FORBIDDEN);
    	}
		else{
	    	//decode the auth header
    		decodeAuthorizationHeader();

        	//authenticate the user against the database
        	oAuthenticationAccount = oHibernateController.authenticateUser(oAuthenticationAccount);

			//check if the authentication failed
			if(oAuthenticationAccount == null){
        		throw new WebApplicationException(Response.Status.UNAUTHORIZED);
        	}
		}

        return createHypermedia(oHibernateController.putrestparameter(oJavarestparameterModel));
    }

	/* This function performs the decoding of the authentication header */
    public void decodeAuthorizationHeader()
    {
    	//check if this request has basic authentication
    	if( !authHeader.contains("Basic "))
    	{
    		throw new WebApplicationException(Response.Status.BAD_REQUEST);
    	}
    	
        authHeader = authHeader.substring("Basic ".length());
        String[] decodedHeader;
        decodedHeader = Base64.base64Decode(authHeader).split(":");
        
        if( decodedHeader == null)
        {
        	throw new WebApplicationException(Response.Status.BAD_REQUEST);
        }
        
        oAuthenticationAccount.setusername(decodedHeader[0]);
        oAuthenticationAccount.setpassword(decodedHeader[1]);
    }

    /* This function produces hypermedia links to be sent to the client so as it will be able to forward the application state in a valid way.*/
    public JavarestparameterModel createHypermedia(JavarestparameterModel oJavarestparameterModel){

        /* Create hypermedia links towards this specific RESTParameter resource. These can be GET, PUT and/or delete depending on what was specified in the service CIM.*/
        oJavarestparameterModel.getlinklist().add(new HypermediaLink(String.format("%s%s", oApplicationUri.getBaseUri(), oApplicationUri.getPath()), "Get the restparameter", "GET", "Sibling"));
        oJavarestparameterModel.getlinklist().add(new HypermediaLink(String.format("%s%s", oApplicationUri.getBaseUri(), oApplicationUri.getPath()), "Update the restparameter", "PUT", "Sibling"));
        oJavarestparameterModel.getlinklist().add(new HypermediaLink(String.format("%s%s", oApplicationUri.getBaseUri(), oApplicationUri.getPath()), "Delete the restparameter", "DELETE", "Sibling"));


        /* Finally, truncate the current URI so as to point to the resource manager of which this resource is related.
        Then create the hypermedia links towards the parent resources.*/
        int iLastSlashIndex = String.format("%s%s", oApplicationUri.getBaseUri(), String.format("%s", oApplicationUri.getPath()).replaceAll("multirestparameter","multirestparameterManager")).lastIndexOf("/");
        oJavarestparameterModel.getlinklist().add(new HypermediaLink(String.format("%s%s", oApplicationUri.getBaseUri(), String.format("%s", oApplicationUri.getPath()).replaceAll("multirestparameter","multirestparameterManager")).substring(0, iLastSlashIndex), "Create a new restparameter", "POST", "Parent"));
        oJavarestparameterModel.getlinklist().add(new HypermediaLink(String.format("%s%s", oApplicationUri.getBaseUri(), String.format("%s", oApplicationUri.getPath()).replaceAll("multirestparameter","multirestparameterManager")).substring(0, iLastSlashIndex), "Get all restparameters of this resource", "GET", "Parent"));
        return oJavarestparameterModel;
    }
}
